# Voting Framework

The voting framework is composed of 4 main processes:
  - The process `Voter_Registration` creates and registers voters.
  - The process `Voter` allows a voter to cast a vote.
  - The process `Cleansing` goes through the ballot box and determines which are valid and should be counted in the final tally
  - The process `Tally` retrieved and outputs the votes from the ballot kept by the `Cleansing` process.

### The Voter Registration process

In our framework, the real identity of a voter is simply a natural number going from 1.
The registration depends on the `letfun` function given in parameter `gen_voter_data(v_id:nat)` which generates the initial private voting data for the voter `v_id`. This private data is sent over a private channel to trusted parties.

Additionally, the process determines whether the voter `v_id` is corrupted, honest without verifying, or honest and verifying.

### The Voter process

The process `Voter` provides a generic structure for the voting procedure. The value of the vote is given by the adversary on a public channel. The choice of the voter is given by a non-deterministic lookup in the table `voting_data` which binds a voter index and their voting data (type `voting_data_type`).

The voting process is then described by a process also given in parameter `Voting_Process`. This process should fully describe the voter casting their vote. It should include the event `Voted`. Ideally, it should be raised just before the last step of the voting process. The process should end by sending potental data on the private channel `res_voting` which is used for passing arguments.

Finally, the `Voter` process let the attacker choose whether this vote was the final vote of the voter. In that case, the voter can perform some last actions described by the process given as paramter `Final_Check_Process`. This process takes as arguments in particular the output of `Voting_Process` input on the channel `res_voting`.

### The Cleansing process

We assume that ballot are added inside the ballot box represented by the table `ballot_box`. How the ballot are added is described in the framework and should be described concurrently to the framework in a process.

The `Cleansing` process takes the ballot in `ballot_box` and determines if they are valid. Additionally, as we consider revote, the cleansing process partitions the set of ballot in `ballot_box` using data of type `voter_ident`. Note that we do not enforce a relation between `voter_ident` (ident) and `voter_id` (index).

The Cleaning process relies on one memory cell `cell_global_bulletin_board` which contains some data of type `global_data_bulletin_board` updated everytime a new ballot is validated. Additionally, it also considers a memory cell for each `voter_ident` partitioning the ballot box. This cell contains some data of type `data_bulletin_board` updated everytime a new ballot associated to `voter_ident` is validated.

The process `Pub_Voter_Ident_Generation` describes the unbounded generation of elements of type `voter_ident`. The exact generation of each element of type `voter_ident` should be described by the process given as parameter `Gen_Voter_Ident_Process`. The latter should output the generated `voter_ident` on a private channel `res_GVI`.

We assume that the `voter_ident` can be extracted using the `letfun` functin given as parameter `get_ident_from_ballot`. This function takes as parameter amongst others the ballot.

We assume that the validity of a ballot can be determined from by the `letfun` function given as parameter `is_valid_ballot`.

Finally, when the ballot is valid, we assume that the value of the new updated `global_data_bulletin_board` and `data_bulletin_board` is determined by the `letfun` functions given as parameter `update_gbl` and `update_ballot`.

Note that `letfun` functions allows to manipulates, terms, event table inserts and table lookup.

### The Tally process

The tally concludes the framework by retrieving the content of the memory cells associated to each `voter_ident` and extracts the vote from it. The extraction is described by the process given as parameter `Decrypt_Ballot_Process` which should output the value of the vote on the channel `res_decrypt`.

### Assumption

We assume that the process `Pub_Voter_Ident_Generation` should generate distinct values everytime (otherwise it's not usefull for partition).

There is no assumption on the order of the processes. In particular, they can be interleaved without restrictions. For example, voter can be registered while the voting is ongoing. Same thing with on-the-fly generation of `public_id_voter`

The event `Link_Idx_Ident` is used to store the relation between the voter index and its corresponding identifier (e.g. its public credential). This event should be included in the `gen_voter_data` function. 

## Summary of the parameters

The processes:

  - `Gen_Voter_Ident_Process`: describes how to generate an `public_id_voter`
  - `Voting_Process`: describes how a voter should cast their vote.
  - `Final_Check_Process`: describes what the voter does after their last vote (e.g. verification)
  - `Decrypt_Ballot_Process`: describes how the value of the ballot is extracted during the tally
  - `Gen_Voting_Data`: describes how the voter's voting data is generated based on their initial knowledge

The `letfun` functions:
  - `is_valid`: defines the set of valid candidates.
  - `gen_voter_data`: describes the initial data securely given to the voter.
  - `get_ident_from_ballot`: describes how to extract the `public_id_voter` from the ballot
  - `is_valid_ballot`: describes when a ballot is valid
  - `update_ballot` and `update_gbl`:  describe how to update the data in the memory cells during the cleansing process (i.e. when accepting and adding a ballot to the bulletin board)
